/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "leddriver6.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define PWM_PERIOD_ERROR   0

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void leddriver6_cfg_setup ( leddriver6_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->pwm = HAL_PIN_NC;
    cfg->scl = HAL_PIN_NC;
    cfg->sda = HAL_PIN_NC;
    
    // Additional gpio pins

    cfg->int_pin = HAL_PIN_NC;
    
	cfg->dev_pwm_base 	  = MODULE_TIM_1;
	cfg->dev_pwm_channel  = TIM_CHANNEL_NC;
	cfg->dev_pwm_polarity = TIM_ACTIVE_LOW;
	cfg->dev_pwm_freq 	  = LEDDRIVER6_DEF_FREQ;
    cfg->i2c_speed = I2C_STANDARD_SPEED; 
    cfg->i2c_address = 0x4D;
}

LEDDRIVER6_RETVAL leddriver6_init ( leddriver6_t *ctx, leddriver6_cfg_t *cfg )
{
    i2c_master_config_t i2c_cfg;
    pwm_config_t pwm_cfg;
    
    pwm_configure_default( &pwm_cfg );

	pwm_cfg.pin      = cfg->pwm;
	pwm_cfg.base     = cfg->dev_pwm_base;
	pwm_cfg.channel  = cfg->dev_pwm_channel;
	pwm_cfg.polarity = cfg->dev_pwm_polarity;
	pwm_cfg.freq_hz  = cfg->dev_pwm_freq;

    ctx->pwm_freq = cfg->dev_pwm_freq;
    ctx->pwm_period = pwm_open( &ctx->pwm, &pwm_cfg );

    if ( ctx->pwm_period == PWM_PERIOD_ERROR )
    {
        return LEDDRIVER6_INIT_ERROR;
    }

    i2c_master_configure_default( &i2c_cfg );
    i2c_cfg.speed    = cfg->i2c_speed;
    i2c_cfg.pins.scl = cfg->scl;
    i2c_cfg.pins.sda = cfg->sda;

    ctx->slave_address = cfg->i2c_address;

    if (  i2c_master_open( &ctx->i2c, &i2c_cfg ) != I2C_SUCCESS )
    {
        return LEDDRIVER6_INIT_ERROR;
    }

    // Input pins

    digital_in_init( &ctx->int_pin, cfg->int_pin );

    return LEDDRIVER6_OK;

}

void leddriver6_set_duty_cycle ( leddriver6_t *ctx, pwm_data_t duty_cycle )
{
    pwm_set_duty( &ctx->pwm, duty_cycle ); 
}

void leddriver6_pwm_stop ( leddriver6_t *ctx )
{
    pwm_stop( &ctx->pwm ); 
}

void leddriver6_pwm_start ( leddriver6_t *ctx )
{
    pwm_start( &ctx->pwm ); 
}

void leddriver6_generic_read ( leddriver6_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len )
{
    i2c_master_transfer_t i2c_transfer;
    uint8_t tx_buf[ 1 ];

    tx_buf [ 0 ] = reg;
    
    i2c_transfer.slave_address = ctx->slave_address;
    i2c_transfer.buffer   = tx_buf;
    i2c_transfer.count    = 1;
    i2c_transfer.end_mode = I2C_RESTART_MODE;

    i2c_master_start( &ctx->i2c );
    i2c_master_write( &ctx->i2c, &i2c_transfer );
    
    i2c_transfer.buffer   = data_buf;
    i2c_transfer.count    = len;
    i2c_transfer.end_mode = I2C_STOP_MODE;

    i2c_master_read( &ctx->i2c, &i2c_transfer );
}

uint16_t leddriver6_get_pg_voltage ( leddriver6_t *ctx )
{
    uint8_t read_reg[ 2 ];
    uint16_t read_data;

    leddriver6_generic_read( ctx, 0x00, read_reg, 2 );

    read_data = read_reg[ 0 ];
    read_data = read_data << 8;
    read_data = read_data | read_reg[ 1 ];

    read_data &= 0x0FFF;

    return read_data;
}

uint8_t leddriver6_get_interrupt_state ( leddriver6_t *ctx )
{
    return digital_in_read( &ctx->int_pin );
}

// ------------------------------------------------------------------------- END

